/*
 * Copyright (c) 2015 Vasa Consulting LLC
 *
 * This file is part of TYPE61CHK.
 *
 * TYPE61CHK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TYPE61CHK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TYPE61CHK.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Take sender address and perform DNSSEC OPENPGPKEY lookup
 */

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sysexits.h>

#include "support.h"

static int		usage(void);

/* invocation */
char *proginv;
char *smtp_username = NULL, *smtp_password = NULL, *smtp_server = NULL;

int
main(int argc, char *argv[])
{
	int		retval = EX_OK;
	int		tmp;

	int		c = 0;
	char   *sender = NULL;

	char   *user = NULL;
	char   *domain = NULL;
	char   *user_hashed;
	char   *openpgpkey = NULL;
	_Bool	encrypt_reply = false;
	char   *msg_file = NULL;
	char   *msg = NULL;
	char   *final_msg = NULL;

	proginv = argv[0];

	while ((c = getopt(argc, argv, "e:hp:s:u:V")) != -1)
	{
		switch (c)
		{
			case 'e':
				smtp_server = optarg;
				break;

			case 'p':
				smtp_password = optarg;
				break;

			case 's':
				sender = optarg;
				break;

			case 'u':
				smtp_username = optarg;
				break;

			case 'V':
				printf("%s: %s v%s\n", argv[0], T61_NAME,
					   T61_VERSION);
				return EX_OK;
				break;

			case 'h':
			default:
				return usage();
				break;
		}
	}

	if (sender == NULL
		|| smtp_server == NULL
		|| smtp_username == NULL
		|| smtp_password == NULL
		|| optind != argc)
		return usage();

	init_curl(smtp_username);
	init_gpgme();
	init_sha();

	/* main */
	if ((retval = dichotomize(sender, &user, &domain)))
		return retval;

	user_hashed = malloc(sizeof(char) * T61_SHA224_BYTES + 1);
	memset(user_hashed, '\0', sizeof(char) * T61_SHA224_BYTES + 1);

	if ((retval = hash_user(user, user_hashed)))
		return retval;

	/*
	 * -3	Other problem
	 * -2	No OPENPGPKEY record
	 * -1	OPENPGPKEY record present and valid
	 *  0	OPENPGPKEY record present and valid, DNSSEC-signed
	 */
	tmp = fetch_openpgpkey(user_hashed, domain, &openpgpkey);

	if (tmp == 0)
	{
		encrypt_reply = true;
		msg_file = "messages/openpgpkey-record-signed.txt";
	}
	else if (tmp == -1)
	{
		encrypt_reply = true;
		msg_file = "messages/openpgpkey-record.txt";
	}
	else if (tmp == -2)
		msg_file = "messages/no-openpgpkey-record.txt";
	else
		retval = EX_SOFTWARE;

	if (retval != EX_SOFTWARE)
	{
		char *query = query_text_ldns();
		msg = prep_msg(msg_file, query);

		if (encrypt_reply)
		{
			final_msg = encrypt_with_gpgme(sender, msg, openpgpkey);

			if (final_msg == NULL)
				final_msg = msg;
		}
		else
			final_msg = msg;

		add_recp_to_curl(sender);

		if (add_msg_to_curl(final_msg, sender))
			return EX_SOFTWARE;

		send_email_via_curl();

		if (final_msg != msg)
			free(final_msg);

		free(msg);
		free(query);
	}

	if (openpgpkey != NULL)
		free(openpgpkey);

	if (user != NULL)
		free(user);

	if (domain != NULL)
		free(domain);

	if (user_hashed != NULL)
		free(user_hashed);

	free_curl();
	free_gpgme();

	return retval;
}

static int
usage()
{
	fprintf(stderr, "Usage: %s [options]\n"
					"\t-e server    \tSMTP server\n"
					"\t-p password  \tSMTP server password\n"
					"\t-s sender    \te-mail address to process\n"
					"\t-u username  \tSMTP server username\n"
					"\t-V           \tprint version and exit\n",
			 proginv);

	return EX_USAGE;
}

